# -*- coding: utf8 -*-
##############################################################################
#
# Copyright (c) 2009 Nexedi SA and Contributors. All Rights Reserved.
#                    Jean-Paul Smets-Solanes <jp@nexedi.com>
#                    Łukasz Nowak <luke@nexedi.com>
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsability of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# garantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

# simple interface which have to be implemented on TradeModelLine,
# Transformation, Pay Sheet Model, etc
from zope.interface import Interface
class ITransformation(Interface):
  """
    Common Interface to implementing quering of Indirect Amount
    Models (TaxModelLine, InvoiceModelLine) shall be based on this interface
  """

  def getAggregatedAmountList(context, **kw):
    """Returns implementation specific AggregatedAmountList of amounts
    generated by set of rules

    context - represents object for which calculation shall happen

    **kw - contains implementations specific parameters

    The method returns an instance of AggregatedAmountList class defined
    at ERP5/AggregatedAmountList.py

    Note: This method shall be linear in case if context is order, line,
    applied rule or movement. In case of built delivery this method shall
    be wise enough to CORRECTLY un-linearise calculation, eg. tax is
    time dependent, paysheet build from invoices.
    """
    pass

  def updateAggregatedAmountList(context, **kw):
    """Updates currently existing movements on delivery

    context - represents object on which update shall happen

    **kw - contains implementations specific parameters

    The method return a dict of list of movements : 'movement_to_add_list' and
    'movement_to_delete_list'.
    """
    pass

